//data_handle.cpp
#include "includes.h"
BITMAP *bSpecTiles[5];
BITMAP *bOtTiles[5], *bEtc[16];
ALFONT_FONT *fonts[3];
MIDI *music;
SAMPLE *sfx[4];

//UTIL FUNCS
int fh = 0, music_on=0, plays=0, mlength; //will keep the value of the font height
char fname[256], buf[256], tmp[64];
char *absolute_path(char *destination){
    get_executable_name(buf, sizeof(buf));
    make_absolute_filename(fname, buf, destination, sizeof(fname));
    return fname;
    }

//DATA HANDLING
  
int load_data(){
    //set all to NULL, just in case
    for(unsigned int k=0; k<(sizeof(bSpecTiles)/sizeof(BITMAP*)); k++)
        bSpecTiles[k] = NULL;
    for(unsigned int k=0; k<(sizeof(bOtTiles)/sizeof(BITMAP*)); k++)
        bOtTiles[k] = NULL;
    for(unsigned int k=0; k<(sizeof(bEtc)/sizeof(BITMAP*)); k++)
        bEtc[k] = NULL;
    for(unsigned int k=0; k<(sizeof(fonts)/sizeof(ALFONT_FONT*)); k++)
        fonts[k] = NULL;
    //LOAD THE DATA
    BITMAP *tmp = load_bitmap(absolute_path("/media/sliding.bmp"), NULL);
    if(tmp==NULL){
        tmp = create_bitmap(192,64); //if the file isn't there invent it
        rectfill(tmp, 0,0, 192, 64, makecol(255,0,0));
        }
    for(int k=0; k<3; k++){ //load the special tiles
        bSpecTiles[k] = create_bitmap(64,64);
        blit(tmp, bSpecTiles[k], k*64, 0,0, 0, 64, 64);
        }
    bSpecTiles[3] = load_bitmap(absolute_path("/media/repulse.bmp"), NULL);
    bSpecTiles[4] = load_bitmap(absolute_path("/media/attract.bmp"), NULL);
    for(int k=3; k<5; k++)
        if(bSpecTiles[k]==NULL){
            bSpecTiles[k]=create_bitmap(64,64); //fill in if the files don't exist
            clear_to_color(bSpecTiles[k], makecol(0,255,0));
            }
    
    bOtTiles[0] = load_bitmap(absolute_path("/media/tile.bmp"), NULL);
    bOtTiles[1] = load_bitmap(absolute_path("/media/tile_low.bmp"), NULL);
    bOtTiles[2] = load_bitmap(absolute_path("/media/tile_top.bmp"), NULL);
    bOtTiles[3] = load_bitmap(absolute_path("/media/hole.bmp"), NULL);
    
    bEtc[0] = load_bitmap(absolute_path("/media/start.bmp"), NULL);
    bEtc[1] = load_bitmap(absolute_path("/media/you.bmp"), NULL);
    bEtc[2] = load_bitmap(absolute_path("/media/granny.bmp"), NULL);
    bEtc[3] = load_bitmap(absolute_path("/media/dialog.bmp"), NULL);
    bEtc[4] = load_bitmap(absolute_path("/media/castle.bmp"), NULL);
    bEtc[5] = load_bitmap(absolute_path("/media/chuck.bmp"), NULL);
    bEtc[6] = load_bitmap(absolute_path("/media/grannie.bmp"), NULL);
    bEtc[7] = load_bitmap(absolute_path("/media/flame0.bmp"), NULL);
    bEtc[8] = load_bitmap(absolute_path("/media/flame1.bmp"), NULL);
    bEtc[9] = load_bitmap(absolute_path("/media/flame2.bmp"), NULL);
    bEtc[10] = load_bitmap(absolute_path("/media/exp3.bmp"), NULL);
    bEtc[11] = load_bitmap(absolute_path("/media/exp2.bmp"), NULL);
    bEtc[12] = load_bitmap(absolute_path("/media/exp1.bmp"), NULL);
    bEtc[13] = load_bitmap(absolute_path("/media/exp0.bmp"), NULL);
    
    //check for unloaded resources
    for(unsigned int k=0; k<(sizeof(bOtTiles)/sizeof(BITMAP*)); k++)
        if(bOtTiles[k]==NULL){
            bOtTiles[k]=create_bitmap(32,32); //fill in if the files don't exist
            clear_to_color(bOtTiles[k], makecol(0,255,255));
            }
    for(unsigned int k=0; k<(sizeof(bEtc)/sizeof(BITMAP*)); k++)
        if(bEtc[k]==NULL){
            bEtc[k]=create_bitmap(32,32); //fill in if the files don't exist
            clear_to_color(bEtc[k], makecol(50,255,255));
            }
    destroy_bitmap(tmp);
    
    fonts[0] = alfont_load_font(absolute_path("/media/puree2.ttf"));
    alfont_set_font_size(fonts[0], 38);
    fh = alfont_get_font_height(fonts[0]);
    
    //load music
    music = load_midi(absolute_path("/media/nbk.mid"));
    if(music){
        music_on = 1;
        mlength = get_midi_length(music);
        play_midi(music, 1);
        }
        
    plays = 1;
    sfx[0] = load_sample(absolute_path("/media/fire.wav"));
    sfx[1] = load_sample(absolute_path("/media/boom.wav"));
    sfx[2] = load_sample(absolute_path("/media/win.wav"));
    sfx[3] = load_sample(absolute_path("/media/sel.wav"));
    return 0;
    }
    
int unload_data(){
    destroy_midi(music);
    for(unsigned int k=0; k<sizeof(sfx)/sizeof(SAMPLE*); k++)
        destroy_sample(sfx[k]);
    
    for(unsigned int k=0; k<(sizeof(bSpecTiles)/sizeof(BITMAP*)); k++)
        destroy_bitmap(bSpecTiles[k]);
    for(unsigned int k=0; k<(sizeof(bOtTiles)/sizeof(BITMAP*)); k++)
        destroy_bitmap(bOtTiles[k]);
    for(unsigned int k=0; k<(sizeof(bEtc)/sizeof(BITMAP*)); k++)
        destroy_bitmap(bEtc[k]);
    for(unsigned int k=0; k<(sizeof(fonts)/sizeof(ALFONT_FONT*)); k++)
        alfont_destroy_font(fonts[k]);
             
    return 0;
    }
    
void draw_score(BITMAP* buf, list<int> *sco, int score){
    list<int>::iterator p;
    rectfill(buf, 180, 10, 460, 50, makecol(90,90,90));
    rectfill(buf, 182, 400, 460, fh*12+16, makecol(0,0,0));
    rectfill(buf, 182, 12, 458, fh*12+14, makecol(190,190,190));
    alfont_textout_centre_aa_ex(buf, fonts[0], "Scores:", screen->w/2, 10, makecol(190,0,0), -1);
    alfont_textout_centre_aa_ex(buf, fonts[0], "Press ENTER key to continue...", screen->w/2, fh*11+10, makecol(190,0,0), -1);
    int k=0;
    for(p = sco->begin(); p!=sco->end(); p++){
        sprintf(tmp, "%2d. Evil Creature:   %4d",k+1,*p);
        if(*p == score)
            alfont_textout_centre_aa_ex(buf, fonts[0], tmp, buf->w/2, 40+k*fh, makecol(190,0,0), -1);
        else 
            alfont_textout_centre_aa_ex(buf, fonts[0], tmp, buf->w/2, 40+k*fh, makecol(0,0,0), -1);
        k++;
        if(k>9)break;
        }
    }
    
void draw_dialog(BITMAP* buf, game *agame){
    int hy = buf->h-bEtc[3]->h;
    draw_sprite(buf, bEtc[3], 0, hy);
    draw_sprite(buf, bEtc[agame->dialogFace], 27, hy+18);
    alfont_textout_aa_ex(buf, fonts[0], agame->dialogText[0], 140, hy+10, makecol(0,0,0), -1);
    alfont_textout_aa_ex(buf, fonts[0], agame->dialogText[1], 140, hy+35, makecol(0,0,0), -1);
    alfont_textout_aa_ex(buf, fonts[0], agame->dialogText[2], 140, hy+60, makecol(0,0,0), -1);
    }
    
void draw(BITMAP *buf, game *agame){
    //DRAW THE BG
    clear_to_color(buf, makecol(60,90,120));
    for(int x=0; x<(buf->w / bOtTiles[1]->w); x++) //lower bit
        draw_sprite(buf,bOtTiles[1], x*bOtTiles[1]->w, buf->h-bOtTiles[1]->h);
        
    for(int x=0; x<(buf->w / bOtTiles[2]->w); x++)//top bit
        draw_sprite(buf,bOtTiles[2], x*bOtTiles[2]->w, 38);
    
    for(int x=0; x<(buf->w / bOtTiles[0]->w); x++) //plain grass
        for(int y=0; y<GRID_Y*2; y++)
            draw_sprite(buf,bOtTiles[0], x*bOtTiles[0]->w, 48+y*bOtTiles[0]->h);
    
    //DRAW THE SPECIAL TILES, IF ANY
    for(int x=0; x<GRID_X; x++)
        for(int y=0; y<GRID_Y; y++)
            if(agame->specialGrid[x][y])
                draw_sprite(buf, bSpecTiles[agame->specialGrid[x][y]-1], x*TILE_S, 48+y*TILE_S);
    
    draw_sprite(buf, bEtc[4], 440, 20);    
    //DRAW THE UI AND GAME OBJECTS
    //draw the meteors
        for(unsigned int k=0; k<agame->flamestrikes.size(); k++){
            draw_sprite(buf, bEtc[7 + agame->flamestrikes[k].frame()], 
                                (int)agame->flamestrikes[k].getX(bEtc[7]->w, bEtc[10]->w), 
                                (int)agame->flamestrikes[k].getY(bEtc[7]->h, bEtc[10]->h));}
        //draw the grannies
        for(unsigned int k=0; k<agame->grannyArmy.size(); k++){
            draw_sprite(buf, bEtc[6], (int)agame->grannyArmy[k].x-bEtc[6]->w/2,
                                    (int)agame->grannyArmy[k].y - bEtc[6]->h);}
        if(agame->mode == S_SETUP){
        //Draw the placing GRID
        for(int x=0; x<=GRID_X; x++)
            vline(buf, x*TILE_S, 48, buf->h-48, makecol(255,255,255));
        for(int y=0; y<=GRID_Y; y++)
            hline(buf, 0, 48+y*TILE_S, GRID_X*TILE_S, makecol(255,255,255));
        //draw the "menu"
        rectfill(buf, buf->w-160, 32, buf->w, buf->h-64, makecol(66,66,66));
        rect(buf, buf->w-160, 32, buf->w, buf->h-64, makecol(30,30,30));
        for(unsigned int k=0; k<(sizeof(bSpecTiles)/sizeof(BITMAP*)); k++)//DRAW THE SPECIAL TILES
            draw_sprite(buf, bSpecTiles[k], buf->w-128, (k+1)*TILE_S);
        for(unsigned int k=0; k<(sizeof(bSpecTiles)/sizeof(BITMAP*)); k++){ //HOW MANY TILES?
            rectfill(buf, buf->w-50, (k+1)*TILE_S+TILE_S/3, buf->w-10, (k+1)*TILE_S+TILE_S*2/3, makecol(200,0,0));
            sprintf(tmp,"%02d",agame->special[k]);
            textout_ex(buf, font, tmp, buf->w-50+16, (k+1)*TILE_S+TILE_S/3+10, makecol(0,0,0), -1);
            }
        draw_sprite(buf, bEtc[0], 522, 424);    
        rect(buf, buf->w-128, TILE_S, buf->w-64, buf->h-96, makecol(230,230,230));
        rect(buf, buf->w-128, (1+agame->selection)*TILE_S, buf->w-64, (2+agame->selection)*TILE_S, makecol(255,0,0));
        }
        
    // ############### IN GAME GRAPHICS ##################
    else if(agame->mode == S_PLAY){
        //draw the UI
        sprintf(tmp,"%03d",agame->shieldSTR);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Shield Strength:", 560+1, 5, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Shield Strength:", 560-1, 5, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Shield Strength:", 560, 5+1, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Shield Strength:", 560, 5-1, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Shield Strength:", 560, 5, makecol(190,0,0), -1);
        
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560+1, fh, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560-1, fh, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560, fh+1, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560, fh-1, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560, fh, makecol(190,0,0), -1);
        
        sprintf(tmp,"%3d",agame->score);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Score:", 560+1, 400, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Score:", 560-1, 400, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Score:", 560, 400+1, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Score:", 560, 400-1, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], "Score:", 560, 400, makecol(190,0,0), -1);
        
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560+1, 395+fh, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560-1, 395+fh, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560, 395+fh+1, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560, 395+fh-1, makecol(0,0,0), -1);
        alfont_textout_centre_aa_ex(buf, fonts[0], tmp, 560, 395+fh, makecol(190,0,0), -1);
        
        }
    else if(agame->mode == S_TALK){
        draw_dialog(buf, agame);
        }
    }

void play(int k){
    if((unsigned)k < sizeof(sfx)/sizeof(SAMPLE*)){
        if(sfx[k])play_sample(sfx[k], 120, 255/2, 1000, 0);
        }
    }
