/*
  Name: gfx.cpp
  Copyright: 2005 FMC
  Author: Fabio Maria Carlucci
  Date: 03/12/05 15.23
  Description: graphical routines related (update, gfx loading, ...) 
*/
#include "includes.h"

static int iPage, iMode, iHasSetup = 0, iX, iY, iFatalError, iColours;
BITMAP *bDbuf, *vPages[3], *bDraw;

int gfx_setup(int mode, int x, int y, int cols){
    iPage = 0;
    iMode = mode;
    iHasSetup = 1;
    iX = x;
    iY = y;
    iFatalError = 0;
    iColours = cols;
    
    return GM_OK;
    }
    
int gfx_start(){
    if(!iHasSetup){
        P_ERROR("You must first setup for gfx!");
        return GM_FAIL;
        }
        
    int iOk;
    set_color_depth(iColours);
    if(iMode & S_FULL)
        iOk = set_gfx_mode(GFX_AUTODETECT, iX, iY, 0, 0);
    else
        iOk = set_gfx_mode(GFX_AUTODETECT_WINDOWED, iX, iY, 0, 0);
        
    if(iOk){
        iMode = 0;
        P_ERROR("\nCan't init selected video mode");
        iFatalError = 1; //fatal error triggered
        return GM_FAIL;
        }
    
    bDraw = screen; //by default the screen is the "drawable" surface
    
    //if triple buffering is enabled
    if(iMode & TRIPLE_BUF){
        /* triple buffering setup */
        if (!(gfx_capabilities & GFX_CAN_TRIPLE_BUFFER))
	       enable_triple_buffer();
	    if (!(gfx_capabilities & GFX_CAN_TRIPLE_BUFFER)) {
            P_ERROR("\nCan't do triple buffering... will try page flipping");
            iMode -= TRIPLE_BUF;
            iMode += PAGE_FLIP;
            }
        else {
    	    vPages[0] = create_video_bitmap(iX, iY);
    	    vPages[1] = create_video_bitmap(iX, iY);
    	    vPages[2] = create_video_bitmap(iX, iY);
            }
        bDraw = vPages[iPage];
        }
    //if page flipping is enabled
    if(iMode & PAGE_FLIP){
        vPages[0] = create_video_bitmap(iX, iY);
    	vPages[1] = create_video_bitmap(iX, iY);
    	bDraw = vPages[iPage];
        }
    //if double buffering is enabled
    if(iMode & DOUBLE_BUF){
        bDbuf = create_bitmap(iX, iY);
        if(!bDbuf){ //if it can't init the double buffer say so and try a workaround
            iMode -= DOUBLE_BUF;
            P_ERROR("\nCan't create double buffer!");
            }
        else
            bDraw = bDbuf; //double buffer can be the drawable bitmap even if page flipping or triple buf are enabled
        }
        
    return GM_OK;
    }
    
int gfx_update(){
    if(iFatalError)return GM_FAIL;
    
    if(iMode & TRIPLE_BUF){
        if(iMode & DOUBLE_BUF)blit(bDbuf, vPages[iPage], 0, 0, 0, 0, screen->w, screen->h);
        
        do {
	    } while (poll_scroll());

	    request_video_bitmap(vPages[iPage]);
	    iPage = (iPage+1)%3;
	    if(!(iMode & DOUBLE_BUF))bDraw = vPages[iPage];
        }
    else if(iMode & PAGE_FLIP){
        if(iMode & DOUBLE_BUF)blit(bDbuf, vPages[iPage], 0, 0, 0, 0, screen->w, screen->h);
        
        show_video_bitmap(vPages[iPage]);
	    iPage = 1-iPage;
	    if(!(iMode & DOUBLE_BUF))bDraw = vPages[iPage];
        }
    else if(iMode & DOUBLE_BUF){
        acquire_screen();
        blit(bDbuf, screen, 0, 0, 0, 0, screen->w, screen->h);
        release_screen();
        }
    
    return GM_OK;
    }
    
int gfx_end(){
    if(iFatalError)return GM_FAIL;
    
    if(iMode & DOUBLE_BUF)destroy_bitmap(bDbuf);
    
    if(iMode & PAGE_FLIP){
        destroy_bitmap(vPages[0]);
        destroy_bitmap(vPages[1]);
        }
    else if(iMode & TRIPLE_BUF){
        destroy_bitmap(vPages[0]);
        destroy_bitmap(vPages[1]);
        destroy_bitmap(vPages[2]);
        }
    
    
    return GM_OK;
    }
